#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.context import register_ext
from pythonite.objects.file_obj import File
import os, re

class CSV(File):
    objtype = 'csv'
    def __init__(self, path, context):
        File.__init__(self, path, context)
        prefix, suffix = os.path.splitext(path)
        self.path = prefix + '.csv'
        file = open(self.path)
        line = file.readline()
        found = re.search(r'[^\w\s"]', line)
        if found:
            self.sep = found.group()
        else:
            self.sep = ','
        word = line.split(self.sep)
        self.desc = [word.strip() for word in word]
        self.lines = file.readlines()
        file.close()
    def select(self, **fields):
        res = []
        fields = fields.items()
        for line in self.lines:
            item = CSVItem(self, line)
            for key, wanted in fields:
                val = getattr(item, key)
                if val != wanted:
                    break
            else:
                res.append(item)
        return res
    def __len__(self):
        return len(self.lines)
    def __getitem__(self, num):
        line = self.lines[num]
        return CSVItem(self, line)
    def __getslice__(self, a, b):
        return [CSVItem(self, line) for line in self.lines[a:b]]

class CSVItem:
    def __init__(self, csv, line):
        self._csv = csv
        self._field = fields = []
        start = -1
        for field in line.rstrip().split(csv.sep):
            fields.append(field)
            if field.endswith('"'):
                if field.startswith('"'):
                    fields[-1] = field[1:-1]
                elif start >= 0:
                    field = csv.sep.join(fields[start:])
                    fields[start:] = [field[1:-1]]
            elif field.startswith('"'):
                start = len(fields) - 1
    def __getattr__(self, attr):
        try:
            i = self._csv.desc.index(attr)
        except ValueError:
            raise AttributeError, attr
        else:
            return self._field[i].strip()

register_ext('.csv', CSV)
