#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.context import register_ext, register_obj
from pythonite.objects.file_obj import File
from pythonite.objects.datetime_obj import DateTime
from pythonite.utils import *
from pythonite.templating import expand
from pythonite import config
import re, os

title_re = re.compile('(<title.*?>.*?</title>)', re.IGNORECASE|re.DOTALL)
body_re = re.compile('<body.*?>(.*)</body>', re.IGNORECASE|re.DOTALL)
imgsrc_re = re.compile('<img src="(?!/|http:)', re.IGNORECASE|re.DOTALL)
meta_re = re.compile('<meta name="(.*?)" content="(.*?)">', re.IGNORECASE|re.DOTALL)
link_re = re.compile('<a href="(.*?)">(.*?)</a>', re.IGNORECASE|re.DOTALL)
href_re = re.compile('href="(.*?)"', re.IGNORECASE|re.DOTALL)

class Link:
    def __init__(self, url, title):
        self.url = url
        self.title = title
    def __str__(self):
        return '<a href="%s">%s</a>' % (self.url, self.title)
    def __repr__(self):
        return "Link('%s')" % self.url

class Meta(Empty):
    def __str__(self):
        lines = []
        for key, val in self.__dict__.items():
            lines.append('<meta name="%s" content="%s">' % (key, val))
        return '\n'.join(lines)

class HTML(File):
    objtype = 'html'
    def get_title(self):
        data = re_find(title_re, self.data)
        if data.find('py:') >= 0:
            data = expand(data, self.context)
        return data[7:-8]
    def get_body(self):
        data = re_find(body_re, self.data)
        if data.find('py:') >= 0:
            data = expand(data, self.context)
        elif self.context.get('document') != self:
            self.url_setup()
            data = href_re.sub(self.url_sub, data)
        return data
    def cut(self, start='', stop=''):
        data = self.data
        if start:
            offset = data.find(start)
            if offset >= 0:
                data = data[offset:]
        if stop:
            offset = data.find(stop)
            if offset >= 0:
                data = data[:offset]
        return data
    def extract(self, tag, offset=0):
        expr = '<%s.*?>(.*?)</%s>(?is)' % (tag, tag)
        data = re.findall(expr, self.data)
        if not data:
            return ''
        return data[offset]
    def url_setup(self):
        olddir = os.path.dirname(self.context.get('request').filename)
        newdir = os.path.dirname(self.path)
        self.prefix = relpath(newdir, olddir)
    def url_sub(self, match):
        url = match.group(1)
        if not url.startswith('/') and not url.startswith('http:'):
            url = os.path.join(self.prefix, url)
        return 'href="%s"' % url
    def get_links(self):
        self.links = []
        for url, title in link_re.findall(self.body):
            self.links.append(Link(url, title))
        return self.links
    def get_meta(self):
        self.meta = Meta()
        for key, val in meta_re.findall(self.data):
            key = key.lower().replace('-','_')
            if key == 'date':
                val = DateTime(date=val)
            setattr(self.meta, key, val)
        return self.meta

class URL(HTML):
    def __init__(self, url, context):
        self.path = url
        self.context = context
        self.data = urllib.urlopen(url).read()

register_ext(config.extension, HTML)
register_obj('document', HTML, 'request.filename')
