#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.context import register_ext
from pythonite.objects.file_obj import File

class MP3(File):
    objtype = 'audio'
    tag = None
    info = None
    def id3tag(self, tag):
        if self.tag == None:
            try:
		import pyid3lib
	        self.tag = pyid3lib.tag(self.path)
	    except:
	        return ''
        try:
	    s = getattr(self.tag, tag)
	    if type(s) == type(''):
		while s.endswith('\x00'):
		    s = s[:-1]
	    return s
	except AttributeError:
	    return ''
    def mp3info(self, func):
        if self.info == None:
            try:
		import mad
	        self.mad = mad.MadFile(self.path)
	    except:
	        return ''
        try:
	    return getattr(self.mad, func)()
	except AttributeError:
	    return ''
    def get_title(self):
        return self.id3tag('title')
    def get_album(self):
        return self.id3tag('album')
    def get_artist(self):
        return self.id3tag('artist')
    def get_year(self):
        return self.id3tag('year')
    def get_track(self):
	try:
            return self.id3tag('tracknum')[0]
	except IndexError:
	    return ''
    def get_duration(self):
        t = self.mp3info('total_time') / 1000
	m, s = divmod(t, 60)
	h, m = divmod(m, 60)
	return '%d:%02d:%02d' % (h, m, s)
    def get_bitrate(self):
        b = self.mp3info('bitrate')
	return '%.0f Kbit/s' % (b/1000.)
    def get_samplerate(self):
        k = self.mp3info('samplerate')
	return '%.1f KHz' % (k/1000.)
    def get_bits(self):
        return 16
    def get_channels(self):
        return 2
    def get_codec(self):
        return 'MP3'

register_ext('.mp3', MP3)
