#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

try:
    from xml.parsers.sgmllib import SGMLParser
except ImportError:
    from sgmllib import SGMLParser
from pythonite.utils import *
import os

class HTMLTableParser(SGMLParser):
    entitydefs = {}
    def __init__(self):
        SGMLParser.__init__(self)
        self.buffer = []
        self.rowbuf = []
        self.rowlevel = 0
    def handle_data(self, data):
        if self.rowlevel:
            self.buffer.append(data)
    def unknown_starttag(self, tag, attrs):
        if self.rowlevel:
            attrs = [' '+key+'="'+val+'"' for (key,val) in attrs]
            self.buffer.append('<'+tag+''.join(attrs)+'>')
        if tag == 'tr':
            self.rowlevel += 1
    def unknown_endtag(self, tag):
        if tag == 'tr':
            self.rowlevel -= 1
            if self.rowlevel == 0:
                self.rowbuf.append(''.join(self.buffer))
                self.buffer = []
        if self.rowlevel:
            self.buffer.append('</'+tag+'>')
    def unknown_entityref(self, entity):
        if self.rowlevel:
            self.buffer.append('&' + entity + ';')

def parse_table(data):
    parser = HTMLTableParser()
    parser.feed(data)
    parser.close()
    # workaround for xml.parsers.sgmllib memory leak
    parser.feed = None
    parser.parser = None
    table = Empty()
    table.row = parser.rowbuf
    return table

class HTMLParser(SGMLParser):
    entitydefs = {}
    def __init__(self, context):
        SGMLParser.__init__(self)
        self.context = context
        self.tagstop = ''
        self.prefix = ''
        self.cmd = ''
        self.buffer = []
    def output(self):
        return ''.join(self.buffer)
    def url_adjust(self, key, val):
        ukey = key.upper()
        if ((ukey == 'HREF' or ukey == 'SRC') and
            not val.startswith('http:') and
            not val.startswith('/')):
            val = self.prefix + val
        return (key, val)
    def get_block(self, full=0, sub=None):
        block = self.block
        if len(block) < 2:
            return ''
        if full and block[-1] == '</span>':
            full = 0
        if sub != None:
            block[1:-1] = [sub]
        if not full:
            block = block[1:-1]
        return ''.join(block)
    def append(self, text):
        if self.tagstop:
            self.block.append(text)
        else:
            self.buffer.append(text)
    def handle_data(self, data):
        self.append(data)
    def write_tag(self, tag, attrs):
        attrs = [' '+key+'="'+val+'"' for (key,val) in attrs]
        self.append('<'+tag+''.join(attrs)+'>')
    def unknown_starttag(self, tag, attrs):
        if self.tagstop:
            if self.tagstop == tag:
                self.tagcount += 1
            self.write_tag(tag, attrs)
            return
        cmd = ''
        attrlist = []
        for key, val in attrs:
            if key.startswith('py:'):
                if key[3:] == 'attr':
                    new = str(self.context.eval(val))
                    attr, old = attrlist[-1]
                    attrlist[-1] = self.url_adjust(attr, new)
                elif key[3:] == 'set':
                    attr, expr = val.split(None, 1)
                    new = self.context.eval(expr)
                    dot = attr.rfind('.')
                    if dot >= 0:
                        obj = self.context.eval(attr[:dot])
                        setattr(obj, attr[dot+1:], new)
                    else:
                        self.context.set(attr, new)
                else:
                    cmd = key[3:]
                    arg = val
            else:
                attrlist.append(self.url_adjust(key, val))
        if cmd:
            self.tagstop = tag
            self.tagcount = 1
            self.cmd = cmd
            self.arg = arg
            self.attrlist = attrlist
            self.block = []
        self.write_tag(tag, attrlist)
    def unknown_endtag(self, tag):
        endtag = '</' + tag + '>'
        self.append(endtag)
        if self.tagstop == tag:
            self.tagcount -= 1
            if self.tagcount <= 0:
                self.tagstop = ''
                if self.cmd == 'sub' or self.cmd == 'rep':
                    if self.arg.startswith('if '):
                        i = self.arg.find(': ')
                        if i >= 0:
                            cond = self.context.eval(self.arg[3:i])
                            if cond:
                                data = str(self.context.eval(self.arg[i+2:]))
                                data = self.get_block(full=(self.cmd == 'sub'), sub=data)
                                self.append(data)
                            else:
                                data = self.get_block(full=(self.cmd == 'sub'))
                                self.append(data)
                        else:
                            cond = self.context.eval(self.arg[3:])
                            if cond:
                                data = self.get_block(full=(self.cmd == 'sub'))
                                data = expand(data, self.context)
                                self.append(data)
                    elif self.arg.startswith('for '):
                        i = self.arg.find(' in ')
                        if i < 0:
                            self.append(str(self.context.eval('*** for without in ***')))
                            self.block = []
                            self.cmd = ''
                            return
                        self.itemname = self.arg[4:i]
                        arg = self.arg[i+4:]
                        i = arg.find(': ')
                        if i >= 0:
                            self.items = self.context.eval(arg[:i])
                            expr = arg[i+2:]
                        else:
                            self.items = self.context.eval(arg)
                            template = self.get_block(full=(self.cmd == 'sub'))
                            expr = ''
                        iter = Empty()
                        save = self.context.save('iteration', iter)
                        try:
                            iter.max = len(self.items)
                        except:
                            iter.max = 0
                        iter.num = 1
                        for item in self.items:
                            iter.first = iter.num == 1
                            iter.last = iter.num == iter.max
                            if iter.num & 1:
                                iter.odd = 1
                                iter.even = 0
                            else:
                                iter.odd = 0
                                iter.even = 1
                            self.context.set(self.itemname, item)
                            if expr:
                                data = str(self.context.eval(expr))
                                if self.cmd == 'sub':
                                    data = self.get_block(full=True, sub=data)
                            else:
                                data = expand(template, self.context)
                            self.append(data)
                            iter.num += 1
                        self.context.restore(save)
                    else:
                        data = str(self.context.eval(self.arg))
                        data = self.get_block(full=(self.cmd == 'sub'), sub=data)
                        self.append(data)
                else:
                    self.append(str(self.context.eval('*** invalid keyword py:%s ***' % self.cmd)))
                self.cmd = ''
                self.block = []
    def unknown_entityref(self, entity):
        self.append('&' + entity + ';')
    def handle_decl(self, data):
        self.append('<!'+data+'>')
    def handle_comment(self, data):
        self.append('<!--'+data+'-->')

def expand(template, context):
    parser = HTMLParser(context)
    parser.feed(template)
    parser.close()
    # workaround for xml.parsers.sgmllib memory leak
    parser.feed = None
    parser.parser = None
    return parser.output()

if __name__ == '__main__':
    from pythonite.context import Context

    # tests
    for template, result in (
        ('<b py:sub="1">0</b>', '<b>1</b>'),
        ('<b py:rep="1">0</b>', '1'),
        ('<b py:sub="if False">0</b>', ''),
        ('<b py:sub="if True">0</b>', '<b>0</b>'),
        ('<b py:sub="if False: 1">0</b>', '<b>0</b>'),
        ('<b py:sub="if True: 1">0</b>', '<b>1</b>'),
        ('<b py:rep="if False">0</b>', ''),
        ('<b py:rep="if True">0</b>', '0'),
        ('<b py:rep="if False: 1">0</b>', '0'),
        ('<b py:rep="if True: 1">0</b>', '1'),
        ('<b py:sub="for i in range(3)">x</b>', '<b>x</b><b>x</b><b>x</b>'),
        ('<b py:rep="for i in range(3)">x</b>', 'xxx'),
        ('<b py:sub="for i in range(3): i">x</b>', '<b>0</b><b>1</b><b>2</b>'),
        ('<b py:rep="for i in range(3): i+1">x</b>', '123'),
        ('<i py:rep="for i in range(1,4)"><b py:rep="for j in range(i): j">x</b>.</i>', '0.01.012.'),
        ('<b py:rep="for x in range(65,68): chr(x)+str(iteration.num)">x</b>', 'A1B2C3'),
        ('<b py:bad="x">x</b>', '?1?'),
        ):
        req = Empty()
        req.errors = []
        context = Context(req)
        context.set('x', 'X')
        html = expand(template, context)
        if html != result: # error
            print template
            print repr(html), '!=', repr(result)
            for err in req.errors:
                print err
            print
